<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8"/>
  <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
  <title>Demo</title>
  <script src="https://cdn.tailwindcss.com"></script>
  <script src="https://unpkg.com/htmx.org@2.0.3"></script>
  <script src="//unpkg.com/alpinejs" defer></script>
</head>
<body hx-headers='{"X-CSRFToken": "{{ csrf_token }}"}'>
{% comment %}
	{% include "partials/navigation.html" %}
{% endcomment %}
<main class="container mx-auto mt-5">
  {% block content %}
  {% endblock content %}
</main>
</body>
</html>
