{% verbatim %}
  {% extends 'base.html' %}

  {% load partials %}

  {% block content %}
  {% endverbatim %}
  <section class="flex justify-between">
    <h1 class="text-lg font-bold">{{ model.verbose_name_plural|capfirst }}</h1>
    <a href="{{ create_view_url|safe }}"
       class="cursor-pointer py-2 px-4 bg-blue-600 hover:bg-blue-700 focus:ring-blue-500 focus:ring-offset-blue-200 text-white text-center text-base font-semibold focus:outline-none focus:ring-2 focus:ring-offset-2 rounded-lg">
      New {{ model.name|capfirst }}
    </a>
  </section>
  {% verbatim %}

    {% partialdef table inline=True %}
      <div id="table">
        <div class="flex flex-col">
          <div class="overflow-x-auto sm:-mx-6 lg:-mx-8">
            <div class="py-2 inline-block min-w-full sm:px-6 lg:px-8">
              <div class="overflow-x-auto">
              {% endverbatim %}
              {{ table_block|safe }}
            </div>
          </div>
        </div>
      </div>
      {{ pagination_block|safe }}
      {% verbatim %}
      </div>
    {% endpartialdef %}

  {% endblock content %}

{% endverbatim %}
